﻿using System;
using System.Windows.Forms;

using Steema.TeeChart;
using Steema.TeeChart.Functions;
using Steema.TeeChart.Styles;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 포인트 시리즈
        /// </summary>
        private Points points;

        /// <summary>
        /// 다운 샘플링 함수
        /// </summary>
        private DownSampling downSampling;

        /// <summary>
        /// 패스트 라인 시리즈
        /// </summary>
        private FastLine fastLine;

        /// <summary>
        /// X 배열
        /// </summary>
        private Nullable<double>[] xArray;

        /// <summary>
        /// Y 배열
        /// </summary>
        private Nullable<double>[] yArray;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 다운 샘플링 메소드 콤보 박스를 설정한다.

            this.downSamplingMethodComboBox.Items.AddRange(Enum.GetNames(typeof(DownSamplingMethod)));

            this.downSamplingMethodComboBox.SelectedIndex = 4;

            #endregion
            #region X/Y 배열을 설정한다.

            int length = 20000;

            xArray = new Nullable<double>[length];
            yArray = new Nullable<double>[length];

            Random random = new Random(DateTime.Now.Millisecond);

            for(int i = 0; i < length; i++)
            {
                xArray[i] = i;

                if(i % 20 == 0)
                {
                    yArray[i] = null;
                }
                else
                {
                    yArray[i] = random.Next(100);
                }
            }

            #endregion
            #region 포인트 시리즈를 생성한다.

            this.points = new Points();

            this.points.Add(this.xArray, this.yArray);

            this.points.Active = false;

            #endregion
            #region 다운 샘플링 함수를 생성한다.

            this.downSampling = new DownSampling();

            this.downSampling.DisplayedPointCount = 1000;
            this.downSampling.Method              = DownSamplingMethod.MinMaxFirstLastNull;

            #endregion
            #region 패스트 라인 시리즈를 생성한다.

            this.fastLine = new FastLine();

            this.fastLine.DataSource = points;
            this.fastLine.Function   = downSampling;
            this.fastLine.TreatNulls = TreatNullsStyle.DoNotPaint;

            #endregion
            #region 티차트를 설정한다.

            this.tChart.Zoom.Direction = ZoomDirections.Horizontal;

            this.tChart.Series.Add(points);
            this.tChart.Series.Add(fastLine);

            #endregion

            this.pointCountNumericUpDown.ValueChanged            += pointCountNumericUpDown_ValueChanged;
            this.downSamplingMethodComboBox.SelectedIndexChanged += downSamplingMethodComboBox_SelectedIndexChanged;
            this.tChart.Zoomed                                   += tChart_Zoomed;
            this.tChart.UndoneZoom                               += tChart_UndoneZoom;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 포인트 수 숫자 UP/DOWN 값 변경시 처리하기 - pointCountNumericUpDown_ValueChanged(sender, e)

        /// <summary>
        /// 포인트 수 숫자 UP/DOWN 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pointCountNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            this.downSampling.DisplayedPointCount = (int)this.pointCountNumericUpDown.Value;

            this.tChart.Axes.Bottom.SetMinMax(0, this.tChart[0].Count);

            this.tChart[1].CheckDataSource();
        }

        #endregion
        #region 다운 샘플링 메소드 콤보 박스 선택 인덱스 변경시 처리하기 - downSamplingMethodComboBox_SelectedIndexChanged(sender, e)

        /// <summary>
        /// 다운 샘플링 메소드 콤보 박스 선택 인덱스 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void downSamplingMethodComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.downSampling.Method = (DownSamplingMethod)Enum.Parse(typeof(DownSamplingMethod), this.downSamplingMethodComboBox.SelectedItem.ToString());

            this.tChart[1].CheckDataSource();
        }

        #endregion
        #region 티차트 확대시 처리하기 - tChart_Zoomed(sender, e)

        /// <summary>
        /// 티차트 확대시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tChart_Zoomed(object sender, EventArgs e)
        {
            this.tChart[1].CheckDataSource();
        }

        #endregion
        #region 티차트 확대 취소시 처리하기 - tChart_UndoneZoom(sender, e)

        /// <summary>
        /// 티차트 확대 취소시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tChart_UndoneZoom(object sender, EventArgs e)
        {
            this.tChart.Axes.Bottom.SetMinMax(0, this.tChart[0].Count);

            this.tChart[1].CheckDataSource();
        }

        #endregion
    }
}